import React, { useEffect } from 'react';
import './App.css';

// Import context providers
import { ServerProvider } from './context/ServerContext/ServerContext';
import { PluginProvider } from './context/PluginContext/PluginContext';
import UIProvider from './context/UIContext/UIContext';
import { useUIContext } from './context/UIContext/useUIContext';

// Import layout components
import Footer from './components/layout/Footer/Footer';
import MainContent from './components/layout/MainContent/MainContent';

// Import server components
import ServerSelector from './components/server/ServerSelector/ServerSelector';
import ServerInfo from './components/server/ServerInfo/ServerInfo';
import ScanProgress from './components/server/ScanProgress/ScanProgress';

// Import plugin components
import PluginList from './components/plugins/PluginList/PluginList';
import PluginDetails from './components/plugins/PluginDetails/PluginDetails';
import NoPluginsMessage from './components/plugins/NoPluginsMessage/NoPluginsMessage';

// Import update components
import UpdateControls from './components/updates/UpdateControls/UpdateControls';
import BulkUpdateProgress from './components/updates/BulkUpdateProgress/BulkUpdateProgress';
import CompatibilityCheckDialog from './components/updates/CompatibilityCheckDialog/CompatibilityCheckDialog';
import PremiumPluginModal from './components/updates/PremiumPluginModal/PremiumPluginModal';
import DownloadProgressIndicator from './components/updates/DownloadProgressIndicator/DownloadProgressIndicator';
import PluginMatchSelector from './components/updates/PluginMatchSelector/PluginMatchSelector';
import WarningModal from './components/updates/WarningModal/WarningModal';

// Import common components
import NotificationDisplay from './components/common/NotificationDisplay/NotificationDisplay';

// Import hooks
import { useServerContext } from './context/ServerContext/useServerContext';
import { usePluginContext } from './context/PluginContext/usePluginContext';
import { usePluginActions } from './hooks/usePluginActions';

/**
 * The main application component that serves as the entry point.
 * This component is responsible for setting up the context providers
 * and rendering the main application structure.
 */
function App() {
  console.log("App Component Initialized");

  return (
    <ServerProvider>
      <PluginContextWrapper />
    </ServerProvider>
  );
}

/**
 * Wrapper to ensure PluginProvider has access to ServerContext
 */
function PluginContextWrapper() {
  const { serverPath, serverInfo } = useServerContext();

  useEffect(() => {
    console.log("PluginContextWrapper: serverPath =", serverPath);
  }, [serverPath]);

  return (
    <PluginProvider>
      <UIProvider>
        <AppContent />
      </UIProvider>
    </PluginProvider>
  );
}

/**
 * The main application content that uses context hooks.
 * This is separate from App to ensure the contexts are available.
 */
function AppContent() {
  const {
    serverInfo,
    serverPath,
    isScanning,
    scanProgress,
    scanComplete
  } = useServerContext();

  const {
    plugins,
    selectedPlugin,
    showPluginDetails,
    closePluginDetails,
    isCheckingUpdates,
    bulkUpdateProgress
  } = usePluginContext();

  const {
    warningMessage,
    clearWarningMessage,
    downloadProgress,
    premiumPluginInfo,
    clearPremiumPluginInfo
  } = useUIContext();

  const { isMatchSelectorOpen, potentialMatches, currentPluginForMatch, handleMatchSelection, closeMatchSelector } = usePluginActions();

  return (
    <div className="app-container">
      <NotificationDisplay />

      <header className="app-header">
        <h1>PlugSnatcher</h1>
        <p className="app-subtitle">Minecraft Server Plugin Manager</p>
      </header>

      <MainContent>
        {/* Server section with selector and info */}
        <div className="server-section">
          <ServerSelector />
          {!isScanning && serverInfo && <ServerInfo serverInfo={serverInfo} />}
        </div>

        {/* Scan progress - only shown when scanning is in progress */}
        {/* Removed duplicate ScanProgress component since it's already in ServerSelector */}

        {/* Plugins section - only shown when scan is complete */}
        {serverPath && scanComplete && plugins.length > 0 && (
          <div className="plugins-section">
            <div className="plugins-header-container">
              <h2>Plugins ({plugins.length})</h2>
              <UpdateControls />
              {bulkUpdateProgress && <BulkUpdateProgress />}
            </div>

            <div className="plugin-grid-header">
              <div>Name</div>
              <div>Version</div>
              <div>Compatibility</div>
              <div>Actions</div>
            </div>

            <PluginList />
          </div>
        )}

        {/* No plugins message - only shown when scan complete but no plugins found */}
        {serverPath && scanComplete && plugins.length === 0 && (
          <NoPluginsMessage />
        )}

        {/* Modals and Overlays */}
        {selectedPlugin && (
          <PluginDetails
            plugin={selectedPlugin}
            onClose={closePluginDetails}
          />
        )}

        {warningMessage && (
          <WarningModal
            isOpen={!!warningMessage}
            onClose={clearWarningMessage}
            title="Warning"
            message={warningMessage.text}
            variant={warningMessage.type === 'error' ? 'error' :
                    warningMessage.type === 'success' ? 'info' : 'warning'}
          />
        )}

        {downloadProgress && downloadProgress > 0 && (
          <DownloadProgressIndicator
            downloadProgress={{
              pluginName: "Plugin",
              version: "Latest",
              percentage: downloadProgress,
              status: "downloading"
            }}
          />
        )}

        {premiumPluginInfo && (
          <PremiumPluginModal
            isOpen={!!premiumPluginInfo}
            onClose={clearPremiumPluginInfo}
            pluginInfo={{
              name: premiumPluginInfo.name,
              version: "Unknown",
              url: "#"
            }}
          />
        )}

{isMatchSelectorOpen && currentPluginForMatch && (
          <PluginMatchSelector
            isOpen={isMatchSelectorOpen}
            onClose={closeMatchSelector}
            pluginName={currentPluginForMatch.name}
            potentialMatches={potentialMatches}
            onSelectMatch={handleMatchSelection}
          />
        )}
      </MainContent>

      <Footer />
    </div>
  );
}

export default App;