import React, { memo, useCallback, KeyboardEvent } from 'react';
import './Badge.css';

export interface BadgeProps {
  /**
   * Badge text content
   */
  label: string;

  /**
   * Badge visual variant/color scheme
   * @default 'default'
   */
  variant?: 'default' | 'primary' | 'success' | 'warning' | 'error' | 'info';

  /**
   * Badge size
   * @default 'medium'
   */
  size?: 'small' | 'medium' | 'large';

  /**
   * Optional icon to display before the label
   */
  icon?: React.ReactNode;

  /**
   * Whether to display badge with an outline style
   * @default false
   */
  outlined?: boolean;

  /**
   * Additional CSS class
   */
  className?: string;

  /**
   * Optional tooltip text displayed on hover
   */
  tooltip?: string;

  /**
   * Optional badge data attribute used for custom styling
   */
  dataAttribute?: {
    name: string;
    value: string;
  };

  /**
   * Click handler for the badge
   */
  onClick?: () => void;
}

/**
 * A reusable badge component for displaying labels, statuses, and categories
 */
const BadgeComponent: React.FC<BadgeProps> = ({
  label,
  variant = 'default',
  size = 'medium',
  icon,
  outlined = false,
  className = '',
  tooltip,
  dataAttribute,
  onClick
}) => {
  const badgeClasses = [
    'app-badge',
    `app-badge-${variant}`,
    `app-badge-${size}`,
    outlined && 'app-badge-outlined',
    onClick && 'app-badge-clickable',
    className
  ].filter(Boolean).join(' ');

  // Construct the data attributes object
  const dataAttrs: { [key: string]: string } = {};
  if (dataAttribute) {
    dataAttrs[`data-${dataAttribute.name}`] = dataAttribute.value;
  }

  const handleKeyDown = useCallback((event: KeyboardEvent<HTMLSpanElement>) => {
    if (onClick && (event.key === 'Enter' || event.key === ' ')) {
      event.preventDefault(); // Prevent default space bar scroll
      onClick();
    }
  }, [onClick]);

  return (
    <span
      className={badgeClasses}
      title={tooltip}
      onClick={onClick}
      onKeyDown={onClick ? handleKeyDown : undefined}
      tabIndex={onClick ? 0 : undefined}
      role={onClick ? 'button' : undefined}
      {...dataAttrs}
    >
      {icon && <span className="app-badge-icon">{icon}</span>}
      <span className="app-badge-label">{label}</span>
    </span>
  );
};

export default memo(BadgeComponent);