import React, { memo } from 'react';
import './Button.css';

export interface ButtonProps extends React.ButtonHTMLAttributes<HTMLButtonElement> {
  /**
   * Button visual variant
   * @default 'primary'
   */
  variant?: 'primary' | 'secondary' | 'outline' | 'danger' | 'success' | 'text' | 'warning';

  /**
   * Button size
   * @default 'medium'
   */
  size?: 'small' | 'medium' | 'large';

  /**
   * Whether the button is in a loading state
   * @default false
   */
  isLoading?: boolean;

  /**
   * Icon to display before the button text
   */
  startIcon?: React.ReactNode;

  /**
   * Icon to display after the button text
   */
  endIcon?: React.ReactNode;

  /**
   * Full width button
   * @default false
   */
  fullWidth?: boolean;

  /**
   * Button content
   */
  children: React.ReactNode;

  /**
   * Optional tooltip text displayed on hover
   */
  tooltip?: string;
}

/**
 * A reusable button component with various styling options
 */
const ButtonComponent: React.FC<ButtonProps> = ({
  variant = 'primary',
  size = 'medium',
  isLoading = false,
  disabled = false,
  startIcon,
  endIcon,
  fullWidth = false,
  className = '',
  tooltip,
  children,
  ...restProps
}) => {
  const buttonClasses = [
    'app-button',
    `app-button-${variant}`,
    `app-button-${size}`,
    isLoading && 'app-button-loading',
    fullWidth && 'app-button-full-width',
    className
  ].filter(Boolean).join(' ');

  return (
    <button
      className={buttonClasses}
      disabled={disabled || isLoading}
      aria-busy={isLoading}
      title={tooltip}
      {...restProps}
    >
      {isLoading && <span role="status" className="app-button-spinner"></span>}
      {startIcon && !isLoading && <span className="app-button-start-icon">{startIcon}</span>}
      <span className="app-button-text">{children}</span>
      {endIcon && !isLoading && <span className="app-button-end-icon">{endIcon}</span>}
    </button>
  );
};

export default memo(ButtonComponent);