import React, { useEffect, useCallback } from 'react';
import { Plugin } from '../../../types/plugin.types';
import { usePluginActions } from '../../../hooks/usePluginActions';
import Badge from '../../common/Badge/Badge';
import Button from '../../common/Button/Button';
import './PluginDetails.css';

interface PluginDetailsProps {
  plugin: Plugin;
  onClose: () => void;
}

// Add a function to create repository URL
const getRepositoryUrl = (plugin: Plugin): string | null => {
  // Return existing repository URL if available
  if (plugin.repository_url) {
    return plugin.repository_url;
  }

  // Try to construct URL based on repository source and ID
  if (plugin.repository_source && plugin.repository_id) {
    switch (plugin.repository_source) {
      case 'SpigotMC':
        return `https://www.spigotmc.org/resources/${plugin.repository_id}`;
      case 'Modrinth':
        return `https://modrinth.com/plugin/${plugin.repository_id}`;
      case 'GitHub':
        return `https://github.com/${plugin.repository_id}/releases`;
      case 'HangarMC':
        return `https://hangar.papermc.io/plugins/${plugin.repository_id}`;
      default:
        return null;
    }
  }

  return null;
};

export const PluginDetails: React.FC<PluginDetailsProps> = ({ plugin, onClose }) => {
  const { updatePlugin, checkSinglePlugin, pluginLoadingStates } = usePluginActions();

  // Check if this specific plugin is being checked or updated
  const isLoading = !!pluginLoadingStates[plugin.file_path];

  // Handle escape key to close modal
  const handleEscapeKey = useCallback((event: KeyboardEvent) => {
    if (event.key === 'Escape') {
      onClose();
    }
  }, [onClose]);

  // Handle clicking outside the modal to close
  const handleOverlayClick = useCallback((event: React.MouseEvent) => {
    if (event.target === event.currentTarget) {
      onClose();
    }
  }, [onClose]);

  // Add event listener for escape key
  useEffect(() => {
    document.addEventListener('keydown', handleEscapeKey);
    return () => {
      document.removeEventListener('keydown', handleEscapeKey);
    };
  }, [handleEscapeKey]);

  const handleCheckUpdate = async () => {
    if (isLoading) return; // Prevent multiple clicks

    try {
      console.log(`Checking for updates for plugin: ${plugin.name}`);
      await checkSinglePlugin(plugin);
    } catch (error) {
      console.error(`Error checking updates for plugin ${plugin.name}:`, error);
    }
  };

  const handleUpdate = async () => {
    if (isLoading) return; // Prevent multiple clicks

    try {
      console.log(`Updating plugin: ${plugin.name}`);
      await updatePlugin(plugin);
      // Note: Any success/error notifications will be handled by the usePluginActions hook
    } catch (error) {
      console.error(`Error updating plugin ${plugin.name}:`, error);
    }
  };

  const formatDate = (timestamp: number | null): string => {
    if (!timestamp) return 'Unknown';
    return new Date(timestamp).toLocaleDateString();
  };

  const renderDependencies = () => {
    const dependencies = [
      ...(plugin.depend || []),
      ...(plugin.soft_depend || [])
    ].filter(Boolean);

    if (dependencies.length === 0) {
      return <p className="no-dependencies">No dependencies</p>;
    }

    return (
      <ul className="dependency-list">
        {dependencies.map((dep, index) => (
          <li key={index}>
            <span className="dependency-name">{dep}</span>
          </li>
        ))}
      </ul>
    );
  };

  const getSizeInMB = (bytes: number | undefined): string => {
    if (!bytes) return 'Unknown';
    return (bytes / (1024 * 1024)).toFixed(2) + ' MB';
  };

  return (
    <div className="plugin-details-overlay" onClick={handleOverlayClick}>
      <div className="plugin-details-container" role="dialog" aria-labelledby="plugin-details-title">
        <div className="plugin-details-header">
          <div>
            <h2 id="plugin-details-title" className="plugin-name">{plugin.name}</h2>
            <div className="plugin-version-info">
              <span className="version">
                Version: {
                  plugin.repository_url || (plugin.repository_source && plugin.repository_id) ? (
                    <a
                      href={getRepositoryUrl(plugin) || '#'}
                      target="_blank"
                      rel="noopener noreferrer"
                      className="version-link"
                    >
                      {plugin.version}
                    </a>
                  ) : plugin.version
                }
              </span>
              {plugin.has_update && plugin.latest_version && (
                <div className="update-info">
                  <Badge
                    label="Update Available"
                    variant="warning"
                    size="small"
                  />
                  <span className="latest-version">
                    Latest: {
                      plugin.repository_url || (plugin.repository_source && plugin.repository_id) ? (
                        <a
                          href={getRepositoryUrl(plugin) || '#'}
                          target="_blank"
                          rel="noopener noreferrer"
                          className="version-link"
                        >
                          {plugin.latest_version}
                        </a>
                      ) : plugin.latest_version
                    }
                  </span>
                </div>
              )}
            </div>
          </div>

          <button
            className="close-button"
            onClick={onClose}
            aria-label="Close details"
          >
            ×
          </button>
        </div>

        <div className="plugin-details-content">
          <div className="plugin-details-section">
            <h3>Description</h3>
            <p className="plugin-description">
              {plugin.description || 'No description available.'}
            </p>
          </div>

          <div className="plugin-details-columns">
            <div className="plugin-details-column">
              <div className="plugin-details-section">
                <h3>Author Information</h3>
                <p className="plugin-authors">
                  {plugin.authors && plugin.authors.length > 0
                    ? plugin.authors.join(', ')
                    : 'Unknown author'}
                </p>
                {plugin.website && (
                  <a
                    href={plugin.website}
                    target="_blank"
                    rel="noopener noreferrer"
                    className="plugin-website"
                  >
                    Visit Website
                  </a>
                )}
              </div>

              <div className="plugin-details-section">
                <h3>Dependencies</h3>
                {renderDependencies()}
              </div>
            </div>

            <div className="plugin-details-column">
              <div className="plugin-details-section">
                <h3>File Information</h3>
                <div className="file-info-item">
                  <span className="file-info-label">Path:</span>
                  <span className="file-info-value file-path">{plugin.file_path}</span>
                </div>
                <div className="file-info-item">
                  <span className="file-info-label">Hash:</span>
                  <span className="file-info-value">{plugin.file_hash.substring(0, 10)}...</span>
                </div>
                {plugin.repository_source && (
                  <div className="file-info-item">
                    <span className="file-info-label">Source:</span>
                    <span className="file-info-value">
                      {plugin.repository_url || getRepositoryUrl(plugin) ? (
                        <a
                          href={getRepositoryUrl(plugin) || '#'}
                          target="_blank"
                          rel="noopener noreferrer"
                        >
                          {plugin.repository_source}
                        </a>
                      ) : plugin.repository_source}
                    </span>
                  </div>
                )}
                {plugin.platform_compatibility && Object.keys(plugin.platform_compatibility).length > 0 && (
                  <div className="file-info-item">
                    <span className="file-info-label">Compatible with:</span>
                    <div className="platform-badges">
                      {Object.entries(plugin.platform_compatibility).map(([platform, isCompatible]) => (
                        <span
                          key={platform}
                          className="platform-badge"
                          data-server-type={platform}
                          title={`${isCompatible ? 'Compatible' : 'May not be compatible'} with ${platform}`}
                        >
                          {platform}
                        </span>
                      ))}
                    </div>
                  </div>
                )}
              </div>
            </div>
          </div>
        </div>

        <div className="plugin-actions-section">
          {plugin.has_update ? (
            <Button
              variant="warning"
              onClick={handleUpdate}
              disabled={isLoading}
              isLoading={isLoading}
              tooltip={`Update ${plugin.name} to version ${plugin.latest_version}`}
              aria-label={`Update ${plugin.name} to version ${plugin.latest_version}`}
            >
              Update to {plugin.latest_version}
            </Button>
          ) : (
            <Button
              variant="outline"
              onClick={handleCheckUpdate}
              disabled={isLoading}
              isLoading={isLoading}
              tooltip={`Check for updates for ${plugin.name}`}
              aria-label={`Check for updates for ${plugin.name}`}
            >
              Check for Updates
            </Button>
          )}
        </div>
      </div>
    </div>
  );
};

export default PluginDetails;