import React, { useCallback, memo } from 'react';
import { Plugin } from '../../../types/plugin.types';
import { usePluginActions } from '../../../hooks/usePluginActions';
import Badge from '../../common/Badge/Badge';
import Button from '../../common/Button/Button';
import './PluginItem.css';

interface PluginItemProps {
  plugin: Plugin;
  onSelect?: (plugin: Plugin) => void;
}

const PluginItemComponent: React.FC<PluginItemProps> = ({ plugin, onSelect }) => {
  const { updatePlugin, checkSinglePlugin, pluginLoadingStates } = usePluginActions();

  // Check if this specific plugin is being checked or updated
  const isLoading = !!pluginLoadingStates[plugin.file_path];

  const handlePluginClick = useCallback(() => {
    if (onSelect) {
      onSelect(plugin);
    }
  }, [onSelect, plugin]);

  const handleUpdateClick = useCallback(async (e: React.MouseEvent) => {
    e.stopPropagation(); // Prevent triggering the parent click
    if (isLoading) return; // Prevent multiple clicks while loading

    try {
      console.log(`Updating plugin: ${plugin.name}`);
      await updatePlugin(plugin);
    } catch (error) {
      console.error(`Error updating plugin ${plugin.name}:`, error);
    }
  }, [updatePlugin, plugin, isLoading]);

  const handleCheckUpdateClick = useCallback(async (e: React.MouseEvent) => {
    e.stopPropagation(); // Prevent triggering the parent click
    if (isLoading) return; // Prevent multiple clicks while loading

    try {
      console.log(`Checking for updates for plugin: ${plugin.name}`);
      await checkSinglePlugin(plugin);
    } catch (error) {
      console.error(`Error checking updates for plugin ${plugin.name}:`, error);
    }
  }, [checkSinglePlugin, plugin, isLoading]);

  const handleInfoClick = useCallback((e: React.MouseEvent) => {
    e.stopPropagation(); // Prevent triggering the parent click
    if (onSelect) {
      console.log(`Viewing details for plugin: ${plugin.name}`);
      onSelect(plugin);
    }
  }, [onSelect, plugin]);

  // Get the plugin compatibility information for display, if available
  const compatibility = plugin.platform_compatibility ? Object.keys(plugin.platform_compatibility).length > 0 : false;

  return (
    <div className={`plugin-item ${plugin.has_update ? 'has-update' : ''}`}>
      {/* Plugin Name Column */}
      <div className="plugin-name-column">
        <h3 className="plugin-name" title={plugin.name}>{plugin.name}</h3>
        {plugin.authors && plugin.authors.length > 0 && (
          <div className="plugin-authors">
            By {plugin.authors.join(', ')}
          </div>
        )}
        {plugin.description && (
          <p className="plugin-description" title={plugin.description}>
            {plugin.description.length > 100
              ? `${plugin.description.substring(0, 100)}...`
              : plugin.description}
          </p>
        )}
      </div>

      {/* Version Column */}
      <div className="plugin-version-column">
        <div className="plugin-version">
          <span className="current-version" title={`Current version: ${plugin.version}`}>v{plugin.version}</span>
          {plugin.has_update && plugin.latest_version && (
            <>
              <span className="version-arrow">→</span>
              <span className="latest-version" title={`Latest version: ${plugin.latest_version}`}>v{plugin.latest_version}</span>
            </>
          )}
        </div>
      </div>

      {/* Compatibility Column */}
      <div className="plugin-compatibility-column">
        {compatibility ? (
          <div className="platform-badges">
            {Object.entries(plugin.platform_compatibility || {}).map(([platform, isCompatible]) => (
              <span
                key={platform}
                className="platform-badge"
                data-server-type={platform}
                title={`${isCompatible ? 'Compatible' : 'May not be compatible'} with ${platform}`}
              >
                {platform}
              </span>
            ))}
          </div>
        ) : (
          <span className="compatibility-unknown">Unknown</span>
        )}
      </div>

      {/* Actions Column */}
      <div className="plugin-actions-column">
        <div className="plugin-actions">
          <Button
            variant="primary"
            size="small"
            onClick={handleInfoClick}
            className="info-button"
            tooltip={`View details for ${plugin.name}`}
            aria-label={`View details for ${plugin.name}`}
          >
            Info
          </Button>

          {plugin.has_update ? (
            <Button
              variant="warning"
              size="small"
              onClick={handleUpdateClick}
              disabled={isLoading}
              isLoading={isLoading}
              className="update-button"
              tooltip={`Update ${plugin.name} to version ${plugin.latest_version}`}
              aria-label={`Update ${plugin.name} to version ${plugin.latest_version}`}
            >
              Update
            </Button>
          ) : (
            <Button
              variant="outline"
              size="small"
              onClick={handleCheckUpdateClick}
              disabled={isLoading}
              isLoading={isLoading}
              className="check-button"
              tooltip={`Check for updates for ${plugin.name}`}
              aria-label={`Check for updates for ${plugin.name}`}
            >
              Check
            </Button>
          )}
        </div>
      </div>
    </div>
  );
};

export const PluginItem = memo(PluginItemComponent);