import React, { useState } from 'react';
import { usePluginActions } from '../../../hooks/usePluginActions';
import { usePluginContext } from '../../../context/PluginContext/usePluginContext';
import { Plugin } from '../../../types/plugin.types';
import { PluginItem } from '../PluginItem/PluginItem';
import Button from '../../common/Button/Button';
import './PluginList.css';

export const PluginList: React.FC = () => {
  const { plugins, isCheckingUpdates } = usePluginActions();
  const { showPluginDetails } = usePluginContext();
  const [searchTerm, setSearchTerm] = useState<string>('');
  const [sortBy, setSortBy] = useState<'name' | 'version' | 'update'>('name');
  const [sortOrder, setSortOrder] = useState<'asc' | 'desc'>('asc');

  // Filter plugins based on search term
  const filteredPlugins = plugins.filter(plugin =>
    plugin.name.toLowerCase().includes(searchTerm.toLowerCase()) ||
    plugin.version.toLowerCase().includes(searchTerm.toLowerCase()) ||
    (plugin.description && plugin.description.toLowerCase().includes(searchTerm.toLowerCase()))
  );

  // Sort plugins based on sort criteria
  const sortedPlugins = [...filteredPlugins].sort((a, b) => {
    let comparison = 0;

    switch (sortBy) {
      case 'name':
        comparison = a.name.localeCompare(b.name);
        break;
      case 'version':
        comparison = a.version.localeCompare(b.version);
        break;
      case 'update':
        // Sort by update status (plugins with updates first)
        comparison = (b.has_update ? 1 : 0) - (a.has_update ? 1 : 0);
        break;
    }

    return sortOrder === 'asc' ? comparison : -comparison;
  });

  const handleSort = (criteria: 'name' | 'version' | 'update') => {
    if (criteria === sortBy) {
      // Toggle sort order if clicking the same criteria
      setSortOrder(prevOrder => prevOrder === 'asc' ? 'desc' : 'asc');
    } else {
      // Set new criteria and default to ascending
      setSortBy(criteria);
      setSortOrder('asc');
    }
  };

  const getSortIndicator = (criteria: 'name' | 'version' | 'update'): string => {
    if (sortBy !== criteria) return '';
    return sortOrder === 'asc' ? ' ↑' : ' ↓';
  };

  return (
    <div className="plugin-list-container">
      <div className="search-container">
        <input
          type="text"
          placeholder="Search plugins..."
          value={searchTerm}
          onChange={(e) => setSearchTerm(e.target.value)}
          className="search-input"
        />
        {searchTerm && (
          <button
            className="clear-search"
            onClick={() => setSearchTerm('')}
            aria-label="Clear search"
          >
            ×
          </button>
        )}
      </div>

      <div className="plugin-list-controls">
        <div className="sort-controls">
          <span className="sort-label">Sort by:</span>
          <Button
            variant="text"
            size="small"
            className={`sort-button ${sortBy === 'name' ? 'active' : ''}`}
            onClick={() => handleSort('name')}
          >
            Name{getSortIndicator('name')}
          </Button>
          <Button
            variant="text"
            size="small"
            className={`sort-button ${sortBy === 'version' ? 'active' : ''}`}
            onClick={() => handleSort('version')}
          >
            Version{getSortIndicator('version')}
          </Button>
          <Button
            variant="text"
            size="small"
            className={`sort-button ${sortBy === 'update' ? 'active' : ''}`}
            onClick={() => handleSort('update')}
          >
            Updates{getSortIndicator('update')}
          </Button>
        </div>

        <div className="filter-info">
          Showing {filteredPlugins.length} of {plugins.length} plugins
        </div>
      </div>

      <div className="plugin-list">
        {sortedPlugins.length > 0 ? (
          sortedPlugins.map(plugin => (
            <PluginItem
              key={plugin.file_path}
              plugin={plugin}
              onSelect={showPluginDetails}
            />
          ))
        ) : (
          <div className="no-results">
            <p>No plugins match your search.</p>
            <Button variant="secondary" onClick={() => setSearchTerm('')}>Clear search</Button>
          </div>
        )}
      </div>
    </div>
  );
};

export default PluginList;