import React from 'react';
import { ServerInfo as ServerInfoType } from '../../../types/server.types';
import Badge from '../../common/Badge/Badge';
import './ServerInfo.css';
// Import only shell open as it's the primary method
import { open } from '@tauri-apps/plugin-shell';
import { useServerContext } from '../../../context/ServerContext/useServerContext';

interface ServerInfoProps {
  serverInfo: ServerInfoType;
}

export const ServerInfo: React.FC<ServerInfoProps> = ({ serverInfo }) => {
  const { serverPath } = useServerContext();

  const getServerTypeIcon = () => {
    switch (serverInfo.server_type.toLowerCase()) {
      case 'paper':
        return '📄';
      case 'spigot':
        return '🔌';
      case 'bukkit':
        return '🪣';
      case 'purpur':
        return '🟣';
      case 'fabric':
        return '🧵';
      case 'forge':
        return '⚒️';
      default:
        return '🖥️';
    }
  };

  const formatServerType = (type: string): string => {
    return type.charAt(0).toUpperCase() + type.slice(1);
  };

  const getStatusColor = (): 'success' | 'warning' | 'error' => {
    const version = serverInfo.minecraft_version;
    if (!version) return 'warning';

    // Parse version numbers like 1.19.2
    const parts = version.split('.');
    const major = parseInt(parts[0] || '0');
    const minor = parseInt(parts[1] || '0');

    // Check if using recent version (arbitrary definition)
    if (major >= 1 && minor >= 18) {
      return 'success';
    } else if (major >= 1 && minor >= 16) {
      return 'warning';
    } else {
      return 'error';
    }
  };

  // Function to open the server directory in file explorer
  const handleOpenDirectory = async () => {
    try {
      console.log('Attempting to open directory using shell.open:', serverPath);
      // Use shell.open directly - it's generally more robust, especially for UNC
      await open(serverPath);
      console.log('Successfully opened directory via shell.open');
    } catch (error) {
      console.error('Failed to open directory:', error);
      // Optionally add user feedback here (e.g., toast notification)
    }
  };

  return (
    <div className="server-info-container">
      <div className="server-info-header">
        <div className="server-icon">{getServerTypeIcon()}</div>
        <div className="server-details">
          <h3 className="server-type-name">
            {formatServerType(serverInfo.server_type)}
            <Badge
              label={serverInfo.minecraft_version || 'Unknown Version'}
              variant={getStatusColor()}
              size="small"
            />
          </h3>
          <p
            className="server-path clickable"
            onClick={handleOpenDirectory}
            title="Click to open folder"
          >
            Server directory <span className="folder-icon">📂</span>
          </p>
        </div>
      </div>

      <div className="server-stats">
        <div className="stat-item">
          <span className="stat-label">Plugins Directory</span>
          <span className="stat-value">{serverInfo.plugins_directory}</span>
        </div>

        <div className="stat-item">
          <span className="stat-label">Plugins Found</span>
          <span className="stat-value">{serverInfo.plugins_count}</span>
        </div>
      </div>
    </div>
  );
};

export default ServerInfo;