import React, { useState } from 'react';
import Modal from '../../common/Modal/Modal';
import Button from '../../common/Button/Button';
import './CompatibilityCheckDialog.css';
import { useServerContext } from '../../../context/ServerContext/useServerContext';
import { Plugin } from '../../../types/plugin.types';

interface CompatibilityCheckDialogProps {
  isOpen: boolean;
  onClose: () => void;
  plugin: Plugin;
  onConfirmUpdate: () => void;
}

/**
 * A dialog that shows compatibility warnings before updating a plugin
 */
export const CompatibilityCheckDialog: React.FC<CompatibilityCheckDialogProps> = ({
  isOpen,
  onClose,
  plugin,
  onConfirmUpdate
}) => {
  const { serverInfo } = useServerContext();
  const [acknowledgedRisk, setAcknowledgedRisk] = useState(false);

  // Determine if the plugin explicitly supports this server version
  const serverVersion = serverInfo?.minecraft_version || '';
  const supportsVersion = plugin.platform_compatibility?.includes(serverVersion) || false;

  const handleConfirm = () => {
    onConfirmUpdate();
    onClose();
  };

  return (
    <Modal
      isOpen={isOpen}
      onClose={onClose}
      title="Compatibility Check"
    >
      <div className="compatibility-check-content">
        <div className="compatibility-status">
          {supportsVersion ? (
            <div className="compatibility-supported">
              <span className="status-icon">✓</span>
              <p>
                This plugin update is marked as compatible with your server version ({serverVersion}).
                It should work correctly after updating.
              </p>
            </div>
          ) : (
            <div className="compatibility-warning">
              <span className="status-icon">⚠️</span>
              <p>
                This plugin update is not explicitly marked as compatible with your server version ({serverVersion}).
                Updating may cause compatibility issues or server errors.
              </p>
            </div>
          )}
        </div>

        <div className="plugin-details">
          <h4>Plugin Details</h4>
          <ul>
            <li><strong>Name:</strong> {plugin.name}</li>
            <li><strong>Current Version:</strong> {plugin.version}</li>
            <li><strong>Latest Version:</strong> {plugin.latest_version}</li>
            {plugin.platform_compatibility && (
              <li>
                <strong>Supported Versions:</strong> {plugin.platform_compatibility.join(', ')}
              </li>
            )}
          </ul>
        </div>

        {!supportsVersion && (
          <div className="risk-acknowledgment">
            <label className="checkbox-container">
              <input
                type="checkbox"
                checked={acknowledgedRisk}
                onChange={() => setAcknowledgedRisk(!acknowledgedRisk)}
              />
              <span className="checkbox-label">
                I understand the risks and still want to update this plugin
              </span>
            </label>
          </div>
        )}

        <div className="dialog-actions">
          <Button
            onClick={onClose}
            variant="outline"
          >
            Cancel
          </Button>
          <Button
            onClick={handleConfirm}
            variant="primary"
            disabled={!supportsVersion && !acknowledgedRisk}
          >
            Update Anyway
          </Button>
        </div>
      </div>
    </Modal>
  );
};

export default CompatibilityCheckDialog;