import React, { useState } from 'react';
import Modal from '../../common/Modal/Modal';
import Button from '../../common/Button/Button';
import { PotentialPluginMatch } from '../../../types/plugin.types';
import './PluginMatchSelector.css';

interface PluginMatchSelectorProps {
  isOpen: boolean;
  onClose: () => void;
  pluginName: string;
  potentialMatches: PotentialPluginMatch[];
  onSelectMatch: (match: PotentialPluginMatch) => void;
}

/**
 * Component that allows users to select from potential plugin matches when the exact match isn't found
 */
export const PluginMatchSelector: React.FC<PluginMatchSelectorProps> = ({
  isOpen,
  onClose,
  pluginName,
  potentialMatches,
  onSelectMatch
}) => {
  const [selectedMatchIndex, setSelectedMatchIndex] = useState<number | null>(null);

  const handleSelectMatch = () => {
    if (selectedMatchIndex !== null) {
      onSelectMatch(potentialMatches[selectedMatchIndex]);
      onClose();
    }
  };

  return (
    <Modal
      isOpen={isOpen}
      onClose={onClose}
      title="Select Plugin Match"
    >
      <div className="plugin-match-selector">
        <div className="plugin-match-header">
          <p>
            We found multiple potential matches for <strong>{pluginName}</strong>.
            Please select the correct plugin from the list below:
          </p>
        </div>

        <div className="plugin-matches-list">
          {potentialMatches.map((match, index) => (
            <div
              key={`${match.repository}-${match.name}-${match.version}`}
              className={`plugin-match-item ${selectedMatchIndex === index ? 'selected' : ''}`}
              onClick={() => setSelectedMatchIndex(index)}
            >
              <div className="plugin-match-info">
                <h4 className="plugin-match-name">{match.name}</h4>
                <span className="plugin-match-version">v{match.version}</span>
                <span className="plugin-match-source">{match.repository}</span>
              </div>

              <div className="plugin-match-details">
                {match.description && (
                  <p className="plugin-match-description">{match.description}</p>
                )}
                <div className="plugin-match-metadata">
                  <span className="plugin-match-mc-versions">
                    MC: {match.minecraft_versions.join(', ')}
                  </span>
                  {match.download_count !== undefined && (
                    <span className="plugin-match-downloads">
                      {new Intl.NumberFormat().format(match.download_count)} downloads
                    </span>
                  )}
                </div>
              </div>

              <div className="plugin-match-selection">
                <div className="plugin-match-radio">
                  <input
                    type="radio"
                    name="plugin-match"
                    checked={selectedMatchIndex === index}
                    onChange={() => setSelectedMatchIndex(index)}
                  />
                </div>
              </div>
            </div>
          ))}
        </div>

        <div className="plugin-match-actions">
          <Button
            onClick={onClose}
            variant="outline"
          >
            Cancel
          </Button>
          <Button
            onClick={handleSelectMatch}
            variant="primary"
            disabled={selectedMatchIndex === null}
          >
            Select Plugin
          </Button>
        </div>
      </div>
    </Modal>
  );
};

export default PluginMatchSelector;