import React, { useState } from 'react';
import { useUpdateActions } from '../../../hooks/useUpdateActions';
import { usePluginActions } from '../../../hooks/usePluginActions';
import Button from '../../common/Button/Button';
import './UpdateControls.css';

export const UpdateControls: React.FC = () => {
  const {
    updateAllPlugins,
    checkForAllUpdates,
    bulkUpdateProgress
  } = useUpdateActions();

  const {
    isCheckingUpdates,
    getOutdatedPluginsCount,
    plugins
  } = usePluginActions();

  // Local loading state for the "Update All" button
  const [isUpdatingAll, setIsUpdatingAll] = useState(false);

  // Calculate how many plugins need updates
  const outdatedCount = getOutdatedPluginsCount();
  const hasUpdates = outdatedCount > 0;

  // Track if updates have been checked (any plugin has has_update property set)
  const updatesHaveBeenChecked = plugins.some(plugin => plugin.has_update !== undefined);

  // Determine overall loading state
  const isLoading = isCheckingUpdates || isUpdatingAll;

  const handleCheckForUpdates = async () => {
    if (isLoading) return;

    try {
      console.log("Starting plugin update check");
      await checkForAllUpdates();
    } catch (error) {
      console.error("Error checking for updates:", error);
    }
  };

  const handleUpdateAll = async () => {
    if (isLoading || !hasUpdates) return;

    try {
      setIsUpdatingAll(true);
      await updateAllPlugins();
    } catch (error) {
      console.error("Error updating all plugins:", error);
    } finally {
      setIsUpdatingAll(false);
    }
  };

  return (
    <div className="update-controls" role="region" aria-label="Plugin update controls">
      <div className="update-status">
        {isCheckingUpdates ? (
          <div className="checking-status" aria-live="polite">
            <div className="spinner small" aria-hidden="true"></div>
            <span>Checking for plugin updates{bulkUpdateProgress ? ` (${bulkUpdateProgress.processed}/${bulkUpdateProgress.total})` : '...'}</span>
          </div>
        ) : hasUpdates ? (
          <div className="updates-available" aria-live="polite">
            <span className="update-count">{outdatedCount}</span> plugin update{outdatedCount !== 1 ? 's' : ''} available
          </div>
        ) : updatesHaveBeenChecked ? (
          <div className="no-updates" aria-live="polite">All plugins are up to date</div>
        ) : (
          <div className="no-updates-checked">Click "Check for Updates" to check plugin versions</div>
        )}
      </div>

      <div className="update-actions">
        <Button
          variant="primary"
          onClick={handleCheckForUpdates}
          disabled={isLoading}
          isLoading={isCheckingUpdates}
          startIcon={!isCheckingUpdates ? <span className="check-icon" aria-hidden="true">↻</span> : undefined}
          aria-label="Check for plugin updates"
        >
          Check for Updates
        </Button>

        {hasUpdates && (
          <Button
            variant="warning"
            onClick={handleUpdateAll}
            disabled={isLoading}
            isLoading={isUpdatingAll}
            startIcon={<span className="update-icon" aria-hidden="true">↑</span>}
            aria-label={`Update all ${outdatedCount} plugins`}
          >
            Update All ({outdatedCount})
          </Button>
        )}
      </div>
    </div>
  );
};

export default UpdateControls;