import React from 'react';
import Modal from '../../common/Modal/Modal';
import Button from '../../common/Button/Button';
import './WarningModal.css';

interface WarningModalProps {
  isOpen: boolean;
  onClose: () => void;
  title: string;
  message: string;
  confirmLabel?: string;
  cancelLabel?: string;
  onConfirm?: () => void;
  variant?: 'warning' | 'error' | 'info';
}

/**
 * A modal component for displaying important warnings to users
 */
export const WarningModal: React.FC<WarningModalProps> = ({
  isOpen,
  onClose,
  title,
  message,
  confirmLabel = 'Confirm',
  cancelLabel = 'Cancel',
  onConfirm,
  variant = 'warning'
}) => {
  const handleConfirm = () => {
    if (onConfirm) {
      onConfirm();
    }
    onClose();
  };

  // Choose icon based on variant
  const renderIcon = () => {
    switch (variant) {
      case 'error':
        return (
          <svg viewBox="0 0 24 24" width="48" height="48" fill="currentColor">
            <path d="M12,2C17.53,2 22,6.47 22,12C22,17.53 17.53,22 12,22C6.47,22 2,17.53 2,12C2,6.47 6.47,2 12,2M15.59,7L12,10.59L8.41,7L7,8.41L10.59,12L7,15.59L8.41,17L12,13.41L15.59,17L17,15.59L13.41,12L17,8.41L15.59,7Z" />
          </svg>
        );
      case 'info':
        return (
          <svg viewBox="0 0 24 24" width="48" height="48" fill="currentColor">
            <path d="M13,9H11V7H13M13,17H11V11H13M12,2A10,10 0 0,0 2,12A10,10 0 0,0 12,22A10,10 0 0,0 22,12A10,10 0 0,0 12,2Z" />
          </svg>
        );
      case 'warning':
      default:
        return (
          <svg viewBox="0 0 24 24" width="48" height="48" fill="currentColor">
            <path d="M13,14H11V10H13M13,18H11V16H13M1,21H23L12,2L1,21Z" />
          </svg>
        );
    }
  };

  return (
    <Modal
      isOpen={isOpen}
      onClose={onClose}
      title={title}
    >
      <div className={`warning-modal warning-modal-${variant}`}>
        <div className="warning-modal-icon">
          {renderIcon()}
        </div>

        <div className="warning-modal-content">
          <p className="warning-modal-message">{message}</p>
        </div>

        <div className="warning-modal-actions">
          {onConfirm ? (
            <>
              <Button
                onClick={onClose}
                variant="outline"
              >
                {cancelLabel}
              </Button>
              <Button
                onClick={handleConfirm}
                variant={variant === 'error' ? 'danger' : 'primary'}
              >
                {confirmLabel}
              </Button>
            </>
          ) : (
            <Button
              onClick={onClose}
              variant="primary"
            >
              OK
            </Button>
          )}
        </div>
      </div>
    </Modal>
  );
};

export default WarningModal;