import React, { createContext, useState, useCallback, ReactNode, useEffect } from 'react';
import { open } from "@tauri-apps/plugin-dialog";
import { invoke } from "@tauri-apps/api/core";
import { listen } from "@tauri-apps/api/event";
import { ServerInfo, ScanProgress, ScanResult } from '../../types/server.types';
import { Plugin } from '../../types/plugin.types';
import { isValidPath } from '../../utils/validators';

interface ServerContextProps {
  /**
   * Current server directory path
   */
  serverPath: string;

  /**
   * Server information (type, version, etc.)
   */
  serverInfo: ServerInfo | null;

  /**
   * Whether a server scan is in progress
   */
  isScanning: boolean;

  /**
   * Whether a scan has been completed
   */
  scanComplete: boolean;

  /**
   * Error message if any operation fails
   */
  error: string | null;

  /**
   * Progress information during scanning
   */
  scanProgress: ScanProgress | null;

  /**
   * Function to select a server directory
   */
  selectDirectory: () => Promise<void>;

  /**
   * Function to scan for plugins in the selected directory
   */
  scanForPlugins: () => Promise<void>;

  /**
   * Function to set the server path directly
   */
  setServerPath: (path: string) => void;

  /**
   * Function to clear any errors
   */
  clearError: () => void;

  /**
   * Function to get the path to the plugins directory
   */
  getPluginsPath: () => string | null;
}

// Create the context with default values
export const ServerContext = createContext<ServerContextProps>({} as ServerContextProps);

interface ServerProviderProps {
  children: ReactNode;
  onScanComplete?: (result: ScanResult) => void;
}

/**
 * Provider component for managing server-related state
 */
export const ServerProvider: React.FC<ServerProviderProps> = ({
  children,
  onScanComplete
}) => {
  const [serverPath, setServerPathState] = useState<string>("");
  const [serverInfo, setServerInfo] = useState<ServerInfo | null>(null);
  const [isScanning, setIsScanning] = useState<boolean>(false);
  const [scanComplete, setScanComplete] = useState<boolean>(false);
  const [error, setError] = useState<string | null>(null);
  const [scanProgress, setScanProgress] = useState<ScanProgress | null>(null);

  // Setup event listeners for scan events
  useEffect(() => {
    const unlisteners: (() => void)[] = [];

    // Handle scan-started event
    listen('scan_started', () => {
      console.log("Received scan_started event in ServerContext");
      setIsScanning(true);
      setScanComplete(false);
      setScanProgress(null);
      setError(null);
    }).then(unlisten => unlisteners.push(unlisten));

    // Handle scan-progress event
    listen('scan_progress', (event) => {
      console.log("Received scan_progress event in ServerContext:", event.payload);
      // Map backend field names to frontend field names
      const progressData = event.payload as { processed: number; total: number; current_file: string };
      setScanProgress({
        current: progressData.processed,
        total: progressData.total,
        current_file: progressData.current_file
      });
    }).then(unlisten => unlisteners.push(unlisten));

    // Handle scan-completed event
    listen('scan_completed', (event) => {
      console.log("Received scan_completed event in ServerContext:", event.payload);
      const result = event.payload as ScanResult;

      setServerInfo(result.server_info);
      setIsScanning(false);
      setScanComplete(true);

      // Call the callback if provided
      if (onScanComplete) {
        onScanComplete(result);
      }
    }).then(unlisten => unlisteners.push(unlisten));

    // Handle scan-error event
    listen('scan_error', (event) => {
      console.log("Received scan_error event in ServerContext:", event.payload);
      setError(event.payload as string);
      setIsScanning(false);
    }).then(unlisten => unlisteners.push(unlisten));

    // Cleanup function to remove event listeners
    return () => {
      unlisteners.forEach(unlisten => unlisten());
    };
  }, [onScanComplete]);

  // Set server path
  const setServerPath = useCallback((path: string) => {
    setServerPathState(path);
    setServerInfo(null);
    setScanComplete(false);
    setError(null);
    setScanProgress(null);
  }, []);

  // Clear error state
  const clearError = useCallback(() => {
    setError(null);
  }, []);

  // Function to select a server directory using Tauri dialog
  const selectDirectory = useCallback(async () => {
    try {
      const selected = await open({
        directory: true,
        multiple: false,
        title: "Select Minecraft Server Folder",
      });

      if (selected && typeof selected === "string") {
        console.log(`Directory selected: ${selected}`);
        setServerPath(selected);

        try {
          // Load persisted plugin data if available
          console.log(`Attempting to load persisted data for: ${selected}`);
          const loadedPlugins: Plugin[] = await invoke("load_plugin_data", { serverPath: selected });

          if (loadedPlugins && loadedPlugins.length > 0) {
            console.log(`Loaded ${loadedPlugins.length} plugins from persistence.`);
            // We don't set plugins here as that's handled by the PluginContext
            setScanComplete(true);
          } else {
            console.log("No persisted plugin data found for this server.");
          }
        } catch (loadError) {
          console.error("Error loading persisted plugin data:", loadError);
          setError(`Failed to load previous plugin data: ${loadError}`);
        }
      } else {
        console.log("Directory selection cancelled.");
      }
    } catch (err) {
      console.error("Error selecting directory:", err);
      setError(`Error selecting directory: ${err}`);
    }
  }, [setServerPath]);

  // Function to scan for plugins in the selected directory
  const scanForPlugins = useCallback(async () => {
    if (!isValidPath(serverPath) || isScanning) return;

    try {
      console.log("Starting scan for plugins in:", serverPath);
      setIsScanning(true);
      setScanComplete(false);
      setScanProgress(null);
      setError(null);

      await invoke("scan_server_dir", { path: serverPath });
      console.log("Scan server dir command invoked successfully");

      // Note: The actual scan results will be received via event listeners
      // which are now set up in the useEffect hook above

    } catch (err) {
      console.error("Error invoking scan command:", err);
      setError(`Failed to start scan: ${err as string}`);
      setIsScanning(false);
    }
  }, [serverPath, isScanning]);

  // Function to get the plugins directory path
  const getPluginsPath = useCallback((): string | null => {
    if (!serverInfo || !serverPath) return null;

    return `${serverPath}/${serverInfo.plugins_directory}`;
  }, [serverPath, serverInfo]);

  // Define the context value
  const contextValue: ServerContextProps = {
    serverPath,
    serverInfo,
    isScanning,
    scanComplete,
    error,
    scanProgress,
    selectDirectory,
    scanForPlugins,
    setServerPath: setServerPathState,
    clearError,
    getPluginsPath
  };

  return (
    <ServerContext.Provider value={contextValue}>
      {children}
    </ServerContext.Provider>
  );
};

export default ServerProvider;