import React, { createContext, useState, useCallback, ReactNode } from 'react';
import { Plugin } from '../../types/plugin.types';

export type MessageType = 'info' | 'warning' | 'error' | 'success';

export interface WarningMessage {
  text: string;
  type: MessageType;
  id: number;
}

export interface PremiumPluginInfo {
  name: string;
  errorMessage: string;
}

export interface UIContextProps {
  warningMessage: WarningMessage | null;
  premiumPluginInfo: PremiumPluginInfo | null;
  downloadProgress: number;
  isCompatibilityDialogOpen: boolean;
  currentPluginForCompatibility: Plugin | null;
  isUpdateAvailable: boolean;
  updateVersion: string | null;
  showWarningMessage: (text: string, type: MessageType) => void;
  clearWarningMessage: () => void;
  handlePremiumPlugin: (errorMessage: string, pluginName: string) => void;
  clearPremiumPluginInfo: () => void;
  setDownloadProgress: (progress: number) => void;
  showCompatibilityDialog: (plugin: Plugin) => void;
  closeCompatibilityDialog: () => void;
  showUpdateAvailableMessage: (version: string) => void;
  clearUpdateAvailableMessage: () => void;
}

export const UIContext = createContext<UIContextProps | null>(null);

interface UIProviderProps {
  children: ReactNode;
}

let messageIdCounter = 0;

export const UIProvider: React.FC<UIProviderProps> = ({ children }) => {
  // State for warning messages
  const [warningMessage, setWarningMessage] = useState<WarningMessage | null>(null);

  // State for premium plugin info
  const [premiumPluginInfo, setPremiumPluginInfo] = useState<PremiumPluginInfo | null>(null);

  // State for download progress
  const [downloadProgress, setDownloadProgress] = useState<number>(0);

  // State for compatibility dialog
  const [isCompatibilityDialogOpen, setIsCompatibilityDialogOpen] = useState<boolean>(false);
  const [currentPluginForCompatibility, setCurrentPluginForCompatibility] = useState<Plugin | null>(null);

  // State for update available notification
  const [isUpdateAvailable, setIsUpdateAvailable] = useState<boolean>(false);
  const [updateVersion, setUpdateVersion] = useState<string | null>(null);

  // Function to show a warning message
  const showWarningMessage = useCallback((text: string, type: MessageType = 'warning') => {
    setWarningMessage({
      text,
      type,
      id: messageIdCounter++
    });

    // Auto-clear messages after 5 seconds except for errors
    if (type !== 'error') {
      setTimeout(() => {
        setWarningMessage(prevMessage =>
          prevMessage && prevMessage.id === messageIdCounter - 1 ? null : prevMessage
        );
      }, 5000);
    }
  }, []);

  // Function to clear the warning message
  const clearWarningMessage = useCallback(() => {
    setWarningMessage(null);
  }, []);

  // Function to handle premium plugin errors
  const handlePremiumPlugin = useCallback((errorMessage: string, pluginName: string) => {
    setPremiumPluginInfo({
      name: pluginName,
      errorMessage
    });
  }, []);

  // Function to clear premium plugin info
  const clearPremiumPluginInfo = useCallback(() => {
    setPremiumPluginInfo(null);
  }, []);

  // Function to show compatibility dialog
  const showCompatibilityDialog = useCallback((plugin: Plugin) => {
    setCurrentPluginForCompatibility(plugin);
    setIsCompatibilityDialogOpen(true);
  }, []);

  // Function to close compatibility dialog
  const closeCompatibilityDialog = useCallback(() => {
    setIsCompatibilityDialogOpen(false);
    setCurrentPluginForCompatibility(null);
  }, []);

  // Function to show update available message
  const showUpdateAvailableMessage = useCallback((version: string) => {
    setUpdateVersion(version);
    setIsUpdateAvailable(true);
    showWarningMessage(`Update available: version ${version}`, 'info');
  }, [showWarningMessage]);

  // Function to clear update available message
  const clearUpdateAvailableMessage = useCallback(() => {
    setIsUpdateAvailable(false);
    setUpdateVersion(null);
  }, []);

  // Context value
  const value: UIContextProps = {
    warningMessage,
    premiumPluginInfo,
    downloadProgress,
    isCompatibilityDialogOpen,
    currentPluginForCompatibility,
    isUpdateAvailable,
    updateVersion,
    showWarningMessage,
    clearWarningMessage,
    handlePremiumPlugin,
    clearPremiumPluginInfo,
    setDownloadProgress,
    showCompatibilityDialog,
    closeCompatibilityDialog,
    showUpdateAvailableMessage,
    clearUpdateAvailableMessage
  };

  return (
    <UIContext.Provider value={value}>
      {children}
    </UIContext.Provider>
  );
};

export default UIProvider;