import { useEffect, useRef } from 'react';
import { listen, UnlistenFn } from "@tauri-apps/api/event";

// Import types
import { ScanProgress, ScanResult } from '../types/server.types';
import { BulkUpdateProgressPayload, SingleUpdateResultPayload, DownloadProgress, PremiumPluginInfo } from '../types/events.types';
import { Plugin } from '../types/plugin.types';

// Event handler types
export type ScanStartedHandler = (serverPath: string) => void;
export type ScanProgressHandler = (progress: { current: number; total: number }) => void;
export type ScanCompletedHandler = (result: {
  plugins: any[];
  server_info: any;
}) => void;
export type ScanErrorHandler = (error: string) => void;

export type BulkUpdateStartHandler = (totalPlugins: number) => void;
export type UpdateCheckProgressHandler = (progress: {
  current: number;
  total: number;
  plugin_name?: string;
}) => void;
export type SingleUpdateStartedHandler = (pluginName: string) => void;
export type SingleUpdateCompletedHandler = (result: {
  success: boolean;
  error?: string;
  plugin_name?: string;
  original_file_path?: string;
}) => void;

export type DownloadProgressHandler = (progress: {
  percentage: number;
  pluginName?: string;
  status: 'downloading' | 'completed' | 'error';
  error?: string;
}) => void;

export type UpdateCheckCompleteHandler = (result: {
  outdated_count: number;
  total_checked: number;
  success: boolean;
}) => void;

export type AppUpdateAvailableHandler = (info: {
  version: string;
  body?: string;
  date?: string;
}) => void;

export interface EventListenerProps {
  // Scan events
  onScanStarted?: ScanStartedHandler;
  onScanProgress?: ScanProgressHandler;
  onScanCompleted?: ScanCompletedHandler;
  onScanError?: ScanErrorHandler;

  // Update check events
  onBulkUpdateStart?: BulkUpdateStartHandler;
  onUpdateCheckProgress?: UpdateCheckProgressHandler;
  onSingleUpdateStarted?: SingleUpdateStartedHandler;
  onSingleUpdateCompleted?: SingleUpdateCompletedHandler;
  onDownloadProgress?: DownloadProgressHandler;
  onUpdateCheckComplete?: UpdateCheckCompleteHandler;

  // App update events
  onAppUpdateAvailable?: AppUpdateAvailableHandler;
}

/**
 * Custom hook for setting up all the Tauri event listeners
 *
 * @param handlers Object containing event handler functions
 */
export const useEventListeners = ({
  onScanStarted,
  onScanProgress,
  onScanCompleted,
  onScanError,
  onBulkUpdateStart,
  onUpdateCheckProgress,
  onSingleUpdateStarted,
  onSingleUpdateCompleted,
  onDownloadProgress,
  onUpdateCheckComplete,
  onAppUpdateAvailable
}: EventListenerProps) => {
  useEffect(() => {
    const unlistenFunctions: UnlistenFn[] = [];

    // Set up scan event listeners
    if (onScanStarted) {
      listen('scan_started', (event) => {
        onScanStarted(event.payload as string);
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    if (onScanProgress) {
      listen('scan_progress', (event) => {
        onScanProgress(event.payload as { current: number; total: number });
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    if (onScanCompleted) {
      listen('scan_completed', (event) => {
        onScanCompleted(event.payload as { plugins: any[]; server_info: any });
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    if (onScanError) {
      listen('scan_error', (event) => {
        onScanError(event.payload as string);
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    // Set up update check event listeners
    if (onBulkUpdateStart) {
      listen('bulk_update_start', (event) => {
        onBulkUpdateStart(event.payload as number);
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    if (onUpdateCheckProgress) {
      listen('update_check_progress', (event) => {
        onUpdateCheckProgress(event.payload as {
          current: number;
          total: number;
          plugin_name?: string;
        });
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    if (onSingleUpdateStarted) {
      listen('single_update_started', (event) => {
        onSingleUpdateStarted(event.payload as string);
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    if (onSingleUpdateCompleted) {
      listen('single_update_completed', (event) => {
        onSingleUpdateCompleted(event.payload as {
          success: boolean;
          error?: string;
          plugin_name?: string;
          original_file_path?: string;
        });
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    if (onDownloadProgress) {
      listen('download_progress', (event) => {
        onDownloadProgress(event.payload as {
          percentage: number;
          pluginName?: string;
          status: 'downloading' | 'completed' | 'error';
          error?: string;
        });
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    if (onUpdateCheckComplete) {
      // Listen for Tauri event
      listen('update_check_complete', (event) => {
        console.log("Received Tauri update_check_complete event:", event.payload);
        onUpdateCheckComplete(event.payload as {
          outdated_count: number;
          total_checked: number;
          success: boolean;
        });
      }).then(unlisten => unlistenFunctions.push(unlisten));

      // Also listen for custom browser event (fallback)
      const handleCustomEvent = (event: any) => {
        console.log("Received custom update_check_complete event:", event.detail);
        onUpdateCheckComplete(event.detail);
      };

      window.addEventListener('update_check_complete', handleCustomEvent);

      // Add cleanup function for browser event
      unlistenFunctions.push(() => {
        window.removeEventListener('update_check_complete', handleCustomEvent);
      });
    }

    if (onAppUpdateAvailable) {
      listen('update_available', (event) => {
        onAppUpdateAvailable(event.payload as {
          version: string;
          body?: string;
          date?: string;
        });
      }).then(unlisten => unlistenFunctions.push(unlisten));
    }

    // Clean up function to remove all listeners
    return () => {
      unlistenFunctions.forEach(unlisten => unlisten());
    };
  }, [
    onScanStarted,
    onScanProgress,
    onScanCompleted,
    onScanError,
    onBulkUpdateStart,
    onUpdateCheckProgress,
    onSingleUpdateStarted,
    onSingleUpdateCompleted,
    onDownloadProgress,
    onUpdateCheckComplete,
    onAppUpdateAvailable
  ]);
};

export default useEventListeners;