import { useCallback, useState, useEffect, useMemo } from 'react';
import { usePluginContext } from '../context/PluginContext/usePluginContext';
import { useUIContext } from '../context/UIContext/useUIContext';
import { useEventListeners } from './useEventListeners';
import { Plugin, PotentialPluginMatch } from '../types/plugin.types';
import { ServerType } from '../types/server.types';
import { isPremiumPluginError } from '../utils/validators';
import { invoke } from '@tauri-apps/api/core';
import { listen } from '@tauri-apps/api/event';

// Define the potential matches payload type
interface PotentialMatchesPayload {
  plugin: {
    file_path: string;
  };
  matches: PotentialPluginMatch[];
}

/**
 * Custom hook that extends PluginContext with enhanced plugin actions
 * and automatic event handling
 */
export const usePluginActions = (serverType?: ServerType) => {
  const {
    plugins,
    selectedPlugin,
    isCheckingUpdates,
    updateError,
    pluginLoadingStates,
    bulkUpdateProgress,
    isCheckingSinglePlugin,
    checkForUpdates: contextCheckForUpdates,
    checkSinglePlugin: contextCheckSinglePlugin,
    updatePlugin: contextUpdatePlugin,
    showPluginDetails,
    closePluginDetails,
    setPlugins,
    clearUpdateError
  } = usePluginContext();

  const {
    showCompatibilityDialog,
    handlePremiumPlugin,
    showWarningMessage
  } = useUIContext();

  // Local state to track updates in progress
  const [updatingPlugin, setUpdatingPlugin] = useState<string | null>(null);
  const [currentUpdateProgress, setCurrentUpdateProgress] = useState<number>(0);

  // Add state for potential matches
  const [potentialMatches, setPotentialMatches] = useState<PotentialPluginMatch[]>([]);
  const [currentPluginForMatch, setCurrentPluginForMatch] = useState<Plugin | null>(null);
  const [isMatchSelectorOpen, setIsMatchSelectorOpen] = useState<boolean>(false);

  // Create a state object for plugin loading states that we manage locally
  const [localPluginLoadingStates, setLocalPluginLoadingStates] = useState<Record<string, boolean>>({});

  // Enhanced check for updates function
  const checkForUpdates = useCallback(async () => {
    await contextCheckForUpdates(serverType);
  }, [contextCheckForUpdates, serverType]);

  // Enhanced function for managing loading states
  const checkSinglePlugin = useCallback(async (plugin: Plugin) => {
    // First, check if the plugin is already being checked
    if (pluginLoadingStates[plugin.file_path] || localPluginLoadingStates[plugin.file_path]) {
      console.log(`Plugin ${plugin.name} is already being checked for updates, skipping.`);
      return;
    }

    // Set loading state before checking update
    setLocalPluginLoadingStates(prev => ({
      ...prev,
      [plugin.file_path]: true
    }));

    try {
      console.log(`Checking plugin ${plugin.name} for updates`);
      await contextCheckSinglePlugin(plugin);
      // The loading state will be cleared by the event listener when update check completes
    } catch (error) {
      console.error(`Error checking plugin ${plugin.name} for updates:`, error);
      showWarningMessage(`Failed to check ${plugin.name} for updates: ${error}`, 'error');

      // Clear loading state on error
      setLocalPluginLoadingStates(prev => {
        const newState = { ...prev };
        delete newState[plugin.file_path];
        return newState;
      });
    }
  }, [contextCheckSinglePlugin, showWarningMessage, pluginLoadingStates, localPluginLoadingStates]);

  // Enhanced update plugin function with compatibility check
  const updatePlugin = useCallback((plugin: Plugin) => {
    // Set loading state
    setLocalPluginLoadingStates(prev => ({
      ...prev,
      [plugin.file_path]: true
    }));

    // Instead of updating directly, show compatibility dialog first
    showCompatibilityDialog(plugin);

    // Note: The actual update will happen when the user confirms in the dialog
    // and the UIContext will call the updatePlugin function from PluginContext
    // Loading state will be cleared by event listeners
  }, [showCompatibilityDialog]);

  // Function to proceed with update after compatibility check
  const proceedWithUpdate = useCallback(async (plugin: Plugin) => {
    try {
      await contextUpdatePlugin(plugin);
    } catch (error: any) {
      console.error('Error in proceedWithUpdate:', error);

      // Handle premium plugin error
      if (isPremiumPluginError(error.toString())) {
        handlePremiumPlugin(error.toString(), plugin.name);
      }
    }
  }, [contextUpdatePlugin, handlePremiumPlugin]);

  // Listen for events
  useEventListeners({
    onBulkUpdateStart: (totalPlugins) => {
      console.log('Bulk update start handled in usePluginActions:', totalPlugins);
      // Context already handles this, no additional action needed
    },
    onUpdateCheckProgress: (progress) => {
      console.log('Update check progress handled in usePluginActions:', progress);
      // Context already handles this, no additional action needed
    },
    onSingleUpdateStarted: (pluginName) => {
      console.log('Single update started handled in usePluginActions:', pluginName);
      // Context already handles this, no additional action needed
    },
    onSingleUpdateCompleted: (result) => {
      console.log('Single update completed handled in usePluginActions:', result);
      const plugin = plugins.find(p => p.file_path === result.original_file_path);
      const pluginName = plugin ? plugin.name : 'Plugin';

      if (result.error) {
        // If it's a premium plugin error, handle it specially
        if (isPremiumPluginError(result.error)) {
          if (plugin) {
            handlePremiumPlugin(result.error, plugin.name);
            // Optionally show a general warning too
            showWarningMessage(`Could not update premium plugin: ${pluginName}. Requires manual download.`, 'warning');
          }
        } else {
          // Show general error notification
          showWarningMessage(`Failed to update ${pluginName}: ${result.error}`, 'error');
        }
      } else {
        // Show success notification
        showWarningMessage(`${pluginName} updated successfully!`, 'success');
      }
    },
    onDownloadProgress: (progress) => {
      console.log('Download progress handled in usePluginActions:', progress);

      if (progress.pluginName) {
        setUpdatingPlugin(progress.pluginName);
        setCurrentUpdateProgress(progress.percentage);

        if (progress.status === 'completed') {
          // Reset after completion
          setTimeout(() => {
            setUpdatingPlugin(null);
            setCurrentUpdateProgress(0);
          }, 1000);
        }
      }
    }
  });

  // Listen for potential matches in a useEffect
  useEffect(() => {
    const unlistenPotentialMatches = listen<PotentialMatchesPayload>('potential_matches_found', (event: { payload: PotentialMatchesPayload }) => {
      const payload = event.payload;
      console.log('Potential matches found:', payload);

      if (payload.matches && payload.matches.length > 0) {
        // Find the plugin this is for
        const targetPlugin = plugins.find(p => p.file_path === payload.plugin.file_path);
        if (targetPlugin) {
          setPotentialMatches(payload.matches);
          setCurrentPluginForMatch(targetPlugin);
          setIsMatchSelectorOpen(true);

          // Clear loading state for this plugin
          setLocalPluginLoadingStates(prev => {
            const newState = { ...prev };
            delete newState[payload.plugin.file_path];
            return newState;
          });
        }
      }
    });

    return () => {
      unlistenPotentialMatches.then((unlisten: () => void) => unlisten());
    };
  }, [plugins]);

  // Function to handle selecting a match
  const handleMatchSelection = useCallback(async (match: PotentialPluginMatch) => {
    if (currentPluginForMatch) {
      try {
        // Set loading state for this plugin
        setLocalPluginLoadingStates(prev => ({
          ...prev,
          [currentPluginForMatch.file_path]: true
        }));

        // Invoke backend to update with selected match
        await invoke('set_plugin_match', {
          pluginPath: currentPluginForMatch.file_path,
          match: {
            name: match.name,
            repository: match.repository,
            repositoryId: match.repository_id,
            version: match.version,
            pageUrl: match.page_url
          }
        });

        // Show success message
        showWarningMessage(`Successfully matched ${currentPluginForMatch.name} with ${match.name} from ${match.repository}`, 'success');

        // Refresh plugin data
        checkSinglePlugin(currentPluginForMatch);
      } catch (error) {
        console.error('Error setting plugin match:', error);
        showWarningMessage(`Failed to set plugin match: ${error}`, 'error');

        // Clear loading state
        setLocalPluginLoadingStates(prev => {
          const newState = { ...prev };
          delete newState[currentPluginForMatch.file_path];
          return newState;
        });
      }
    }

    // Close the selector
    setIsMatchSelectorOpen(false);
    setPotentialMatches([]);
    setCurrentPluginForMatch(null);
  }, [currentPluginForMatch, showWarningMessage, checkSinglePlugin]);

  // Function to close match selector without selecting
  const closeMatchSelector = useCallback(() => {
    setIsMatchSelectorOpen(false);
    setPotentialMatches([]);
    setCurrentPluginForMatch(null);

    // Clear loading state if there's a current plugin
    if (currentPluginForMatch) {
      setLocalPluginLoadingStates(prev => {
        const newState = { ...prev };
        delete newState[currentPluginForMatch.file_path];
        return newState;
      });
    }
  }, [currentPluginForMatch]);

  // Get outdated plugins count
  const getOutdatedPluginsCount = useCallback((): number => {
    return plugins.filter(plugin => plugin.has_update).length;
  }, [plugins]);

  // Get plugin by name
  const getPluginByName = useCallback((name: string): Plugin | undefined => {
    return plugins.find(plugin => plugin.name.toLowerCase() === name.toLowerCase());
  }, [plugins]);

  // Get plugins filtered by criteria
  const getFilteredPlugins = useCallback((criteria: Partial<Plugin>): Plugin[] => {
    return plugins.filter(plugin => {
      return Object.entries(criteria).every(([key, value]) => {
        return plugin[key as keyof Plugin] === value;
      });
    });
  }, [plugins]);

  // Combined loading states from context and local state
  const combinedLoadingStates = useMemo(() => {
    return {
      ...localPluginLoadingStates,
      ...pluginLoadingStates
    };
  }, [localPluginLoadingStates, pluginLoadingStates]);

  // Set up event listeners for plugin-related events
  useEffect(() => {
    const unlisteners: (() => void)[] = [];

    // Listen for bulk updates complete
    window.addEventListener('update_check_complete', (event: any) => {
      console.log('Custom update_check_complete event received:', event.detail);

      if (event.detail?.success && event.detail?.outdated_count) {
        const message = `Found ${event.detail.outdated_count} plugins with updates available.`;
        showWarningMessage(message, 'info');
      }
    });

    // Listen for single plugin update completed
    window.addEventListener('single_plugin_updated', (event: any) => {
      console.log('Custom single_plugin_updated event received:', event.detail);

      if (event.detail?.success) {
        const message = `Successfully updated ${event.detail.plugin_name}.`;
        showWarningMessage(message, 'success');

        // Clear loading state for this plugin
        setLocalPluginLoadingStates(prev => {
          const newState = { ...prev };
          if (event.detail.old_path) {
            delete newState[event.detail.old_path];
          }
          return newState;
        });
      }
    });

    // Listen for single plugin update failed
    window.addEventListener('single_plugin_update_failed', (event: any) => {
      console.log('Custom single_plugin_update_failed event received:', event.detail);

      if (event.detail?.plugin_name) {
        const message = `Failed to update ${event.detail.plugin_name}: ${event.detail.error || 'Unknown error'}`;
        showWarningMessage(message, 'error');

        // Clear loading state for this plugin
        setLocalPluginLoadingStates(prev => {
          const newState = { ...prev };
          if (event.detail.plugin_name) {
            // Find the plugin by name and clear its loading state
            const plugin = plugins.find(p => p.name === event.detail.plugin_name);
            if (plugin?.file_path) {
              delete newState[plugin.file_path];
            }
          }
          return newState;
        });
      }
    });

    // Listen for single plugin check completed
    window.addEventListener('single_plugin_check_completed', (event: any) => {
      console.log('Custom single_plugin_check_completed event received:', event.detail);

      if (event.detail?.plugin_path) {
        // Clear loading state for this plugin
        setLocalPluginLoadingStates(prev => {
          const newState = { ...prev };
          delete newState[event.detail.plugin_path];
          return newState;
        });

        // Show result notification
        if (event.detail.error) {
          showWarningMessage(
            `Failed to check for updates: ${event.detail.error}`,
            'error'
          );
        }
      }
    });

    return () => {
      // Remove event listeners
      window.removeEventListener('update_check_complete', () => {});
      window.removeEventListener('single_plugin_updated', () => {});
      window.removeEventListener('single_plugin_update_failed', () => {});
      window.removeEventListener('single_plugin_check_completed', () => {});
    };
  }, [plugins, showWarningMessage]);

  return {
    // Original context values
    plugins,
    selectedPlugin,
    isCheckingUpdates,
    updateError,
    pluginLoadingStates: combinedLoadingStates,
    bulkUpdateProgress,
    isCheckingSinglePlugin,
    showPluginDetails,
    closePluginDetails,
    setPlugins,
    clearUpdateError,

    // Enhanced functionality
    updatingPlugin,
    currentUpdateProgress,
    checkForUpdates,
    checkSinglePlugin,
    updatePlugin,
    proceedWithUpdate,
    getOutdatedPluginsCount,
    getPluginByName,
    getFilteredPlugins,

    // Add match selector state and handlers
    potentialMatches,
    currentPluginForMatch,
    isMatchSelectorOpen,
    handleMatchSelection,
    closeMatchSelector
  };
};

export default usePluginActions;