import { useState, useCallback } from 'react';
import { useServerContext } from '../context/ServerContext/useServerContext';
import { useEventListeners } from './useEventListeners';
import { ScanResult, ServerInfo } from '../types/server.types';
import { Plugin } from '../types/plugin.types';

/**
 * Custom hook that extends ServerContext with enhanced scan functionality
 * with automatic event handling
 */
export const useServerActions = (onPluginsLoaded?: (plugins: Plugin[]) => void) => {
  const {
    serverPath,
    serverInfo,
    isScanning,
    scanComplete,
    error,
    scanProgress,
    scanForPlugins: contextScanForPlugins,
    selectDirectory: contextSelectDirectory,
    setServerPath,
    clearError,
    getPluginsPath
  } = useServerContext();

  // Local state for enhanced functionality
  const [lastScanResult, setLastScanResult] = useState<ScanResult | null>(null);

  // Handle scan events
  useEventListeners({
    onScanStarted: () => {
      console.log('Scan started handled in useServerActions');
      // This is already handled by the context, no additional action needed
    },
    onScanProgress: (progress) => {
      console.log('Scan progress handled in useServerActions:', progress);
      // This is already handled by the context, no additional action needed
    },
    onScanCompleted: (result) => {
      console.log('Scan completed handled in useServerActions:', result);
      setLastScanResult(result);

      // If a callback was provided, call it with the plugins
      if (onPluginsLoaded) {
        onPluginsLoaded(result.plugins);
      }
    },
    onScanError: (errorMessage) => {
      console.log('Scan error handled in useServerActions:', errorMessage);
      // This is already handled by the context, no additional action needed
    }
  });

  // Enhanced select directory with post-selection actions
  const selectDirectory = useCallback(async () => {
    await contextSelectDirectory();
    // The rest is handled by the context and event listeners
  }, [contextSelectDirectory]);

  // Enhanced scan for plugins with additional options
  const scanForPlugins = useCallback(async () => {
    // Reset the last scan result if we're starting a new scan
    setLastScanResult(null);

    // Call the context's scan function
    await contextScanForPlugins();
    // The rest is handled by the context and event listeners
  }, [contextScanForPlugins]);

  // Get server name based on server info
  const getServerName = useCallback((): string => {
    if (!serverInfo) return 'Unknown Server';

    let name = serverInfo.server_type;
    if (serverInfo.minecraft_version) {
      name += ` (${serverInfo.minecraft_version})`;
    }

    return name;
  }, [serverInfo]);

  // Check if server is compatible with a specific server type
  const isCompatibleWith = useCallback((serverTypes: string[]): boolean => {
    if (!serverInfo) return false;
    return serverTypes.includes(serverInfo.server_type);
  }, [serverInfo]);

  return {
    // Original context values
    serverPath,
    serverInfo,
    isScanning,
    scanComplete,
    error,
    scanProgress,
    setServerPath,
    clearError,
    getPluginsPath,

    // Enhanced functionality
    lastScanResult,
    selectDirectory,
    scanForPlugins,
    getServerName,
    isCompatibleWith
  };
};

export default useServerActions;