import { useCallback } from 'react';
import { usePluginContext } from '../context/PluginContext/usePluginContext';
import { useUIContext } from '../context/UIContext/useUIContext';
import { useEventListeners } from './useEventListeners';
import { useServerContext } from '../context/ServerContext/useServerContext';
import { Plugin } from '../types/plugin.types';

/**
 * Custom hook for managing plugin updates
 */
export const useUpdateActions = () => {
  const {
    plugins,
    checkForUpdates: checkForPluginUpdates,
    updatePlugin,
    bulkUpdateProgress,
    isCheckingUpdates
  } = usePluginContext();

  const {
    showWarningMessage
  } = useUIContext();

  const { serverPath } = useServerContext();

  // Listen for update-related events
  useEventListeners({
    onUpdateCheckComplete: (result) => {
      console.log('Update check complete handled in useUpdateActions:', result);

      if (result.outdated_count > 0) {
        showWarningMessage(
          `Found ${result.outdated_count} plugin(s) with updates available.`,
          'info'
        );
      } else {
        showWarningMessage('All plugins are up to date!', 'success');
      }
    }
  });

  // Update all plugins with available updates
  const updateAllPlugins = useCallback(async () => {
    const pluginsToUpdate = plugins.filter(plugin => plugin.has_update);

    if (pluginsToUpdate.length === 0) {
      showWarningMessage('No plugins with updates available.', 'info');
      return;
    }

    // The actual update process is handled by the PluginContext
    // This is just a convenience function to find and trigger updates
    for (const plugin of pluginsToUpdate) {
      try {
        await updatePlugin(plugin);
      } catch (error: any) {
        console.error(`Error updating plugin ${plugin.name}:`, error);
        // Continue with other plugins if one fails
      }
    }
  }, [plugins, updatePlugin, showWarningMessage]);

  // Check for plugin updates with improved debugging
  const checkForAllUpdates = useCallback(async () => {
    console.log('Starting plugin update check in useUpdateActions');

    // Get the latest server path directly from context
    const currentServerPath = serverPath;

    console.log('Current state:', {
      serverPath: currentServerPath,
      pluginsCount: plugins.length,
      isCheckingUpdates
    });

    if (!currentServerPath) {
      console.error('No server path available. Please select a server first.');
      showWarningMessage('No server path available. Please select a server first.', 'error');
      return;
    }

    if (!plugins.length) {
      console.error('No plugins available to check for updates.');
      showWarningMessage('No plugins available to check for updates.', 'warning');
      return;
    }

    if (isCheckingUpdates) {
      console.log('Update check already in progress, ignoring request');
      return;
    }

    try {
      console.log('Calling checkForPluginUpdates with serverPath:', currentServerPath);
      await checkForPluginUpdates();
      console.log('Plugin update check completed successfully');
    } catch (error) {
      console.error('Error checking for plugin updates:', error);
      showWarningMessage(`Failed to check for updates: ${String(error)}`, 'error');
    }
  }, [checkForPluginUpdates, plugins.length, serverPath, isCheckingUpdates, showWarningMessage]);

  return {
    // Plugin update state from context
    bulkUpdateProgress,

    // Functions
    updateAllPlugins,
    checkForAllUpdates
  };
};

export default useUpdateActions;