declare module '@tauri-apps/api/shell' {
  /**
   * Opens a path or URL with the default application.
   * @param path The path or URL to open.
   * @returns A promise that resolves when the command finishes.
   */
  export function open(path: string): Promise<void>;
}

declare module '@tauri-apps/api/core' {
  /**
   * Invokes a Tauri command.
   * @param cmd The command name.
   * @param args Command arguments.
   * @returns A promise resolving to the command result.
   */
  export function invoke<T = any>(cmd: string, args?: Record<string, unknown>): Promise<T>;
}

declare module '@tauri-apps/api/core/updater' {
  export interface UpdateResult {
    available: boolean;
    manifest?: {
      version: string;
      date: string;
      body: string;
    };
    shouldUpdate: boolean;
  }

  /**
   * Checks if an update is available.
   * @returns A promise resolving to the update check result.
   */
  export function checkUpdate(): Promise<UpdateResult>;

  /**
   * Installs the available update.
   * @returns A promise that resolves when the update is installed.
   */
  export function installUpdate(): Promise<void>;
}

declare module '@tauri-apps/api/core/path' {
  /**
   * Returns the path to the application data directory.
   * @returns A promise resolving to the app data directory path.
   */
  export function appDataDir(): Promise<string>;

  /**
   * Returns the path to the application configuration directory.
   * @returns A promise resolving to the app config directory path.
   */
  export function appConfigDir(): Promise<string>;

  /**
   * Returns the path to the application local data directory.
   * @returns A promise resolving to the app local data directory path.
   */
  export function appLocalDataDir(): Promise<string>;

  /**
   * Returns the path to the application cache directory.
   * @returns A promise resolving to the app cache directory path.
   */
  export function appCacheDir(): Promise<string>;

  /**
   * Returns the path to the application log directory.
   * @returns A promise resolving to the app log directory path.
   */
  export function appLogDir(): Promise<string>;
}

declare module '@tauri-apps/api/core/event' {
  export interface Event<T> {
    payload: T;
    windowLabel?: string;
    event: string;
  }

  export type EventCallback<T> = (event: Event<T>) => void;
  export type UnlistenFn = () => void;

  /**
   * Listen to an event from the backend.
   * @param event The event name.
   * @param handler The event handler.
   * @returns A promise resolving to a function to unlisten to the event.
   */
  export function listen<T>(event: string, handler: EventCallback<T>): Promise<UnlistenFn>;

  /**
   * Listen to an event from the backend once.
   * @param event The event name.
   * @param handler The event handler.
   * @returns A promise resolving to a function to unlisten to the event.
   */
  export function once<T>(event: string, handler: EventCallback<T>): Promise<UnlistenFn>;
}

// Keep the old path declarations for backward compatibility
declare module '@tauri-apps/api/event' {
  export * from '@tauri-apps/api/core/event';
}

declare module '@tauri-apps/api/updater' {
  export * from '@tauri-apps/api/core/updater';
}

declare module '@tauri-apps/api/path' {
  export * from '@tauri-apps/api/core/path';
}