export interface UpdateInfo {
  available: boolean;
  version?: string;
  body?: string;
  date?: string;
}

// Environment detection - check if running in a Tauri environment
const isTauriApp = window && typeof window !== 'undefined' && 'window.__TAURI__' in window;

/**
 * Checks if there's an app update available
 * @returns Promise with update information
 */
export async function checkForAppUpdate(): Promise<UpdateInfo> {
  if (!isTauriApp) {
    console.log('Update check skipped - not running in Tauri environment');
    return { available: false };
  }

  try {
    // Dynamically import the module only when in Tauri context
    // We need to use a variable for the import path to prevent Vite from trying to resolve it at build time
    const updaterModule = '@tauri-apps/api/updater';
    const { checkUpdate } = await import(/* @vite-ignore */ updaterModule);
    const response = await checkUpdate();

    return {
      available: response.available,
      version: response.manifest?.version,
      body: response.manifest?.body,
      date: response.manifest?.date
    };
  } catch (error) {
    console.error('Error checking for update:', error);
    return { available: false };
  }
}

/**
 * Installs the available app update
 * Note: This will restart the app if successful
 * @returns Promise that resolves when update is complete or fails
 */
export async function installAppUpdate(): Promise<void> {
  if (!isTauriApp) {
    console.log('Update installation skipped - not running in Tauri environment');
    return;
  }

  try {
    // Dynamically import the module only when in Tauri context
    const updaterModule = '@tauri-apps/api/updater';
    const { installUpdate } = await import(/* @vite-ignore */ updaterModule);
    await installUpdate();
    // The app will be restarted by Tauri if successful
  } catch (error) {
    console.error('Error installing update:', error);
    throw error;
  }
}