import { ServerType } from '../types/server.types';
import { Plugin } from '../types/plugin.types';

/**
 * Formats a list of authors into a comma-separated string
 * @param authors Array of author names
 * @returns Formatted authors string
 */
export function formatAuthors(authors: string[]): string {
  if (!authors || authors.length === 0) {
    return 'Unknown';
  }

  return authors.join(', ');
}

/**
 * Formats platform compatibility information into a user-friendly label
 * @param plugin The plugin object
 * @param currentServerType The current server type
 * @returns A formatted compatibility string
 */
export function getPlatformCompatibilityLabel(plugin: Plugin, currentServerType?: ServerType): string {
  const platformCompatibility = plugin.platform_compatibility || [];
  if (platformCompatibility.length === 0) {
    return '';
  }

  // Check if current server type is in the compatibility list
  const isCompatible = currentServerType &&
    platformCompatibility.includes(currentServerType);

  if (isCompatible) {
    return `Compatible with your ${currentServerType} server`;
  } else if (currentServerType) {
    return `Compatible with ${platformCompatibility.join(', ')} (You're using ${currentServerType})`;
  } else {
    return `Compatible with ${platformCompatibility.join(', ')}`;
  }
}

/**
 * Creates a status message for plugin updates
 * @param pluginName Name of the plugin
 * @param version Version being updated to
 * @param platformCompatibility Array of compatible platforms
 * @returns Formatted update message
 */
export function createUpdateMessage(pluginName: string, version?: string, platformCompatibility?: string[]): string {
  if (!version) {
    return `Updating ${pluginName}`;
  }

  let updateMessage = `Updating ${pluginName} to version ${version}`;

  if (platformCompatibility && platformCompatibility.length > 0) {
    updateMessage += ` (${platformCompatibility.join(', ')} compatible)`;
  }

  return updateMessage;
}