import { Plugin } from '../types/plugin.types';

/**
 * Check if a plugin can be updated
 * @param plugin The plugin to check
 * @returns True if the plugin can be updated, false otherwise
 */
export function canUpdatePlugin(plugin: Plugin): boolean {
  return !!(
    plugin.has_update &&
    plugin.latest_version &&
    (plugin.repository_source || plugin.repository_id)
  );
}

/**
 * Check if a plugin is from a premium source that requires manual download
 * @param errorMessage The error message to check
 * @returns True if the plugin is premium, false otherwise
 */
export function isPremiumPluginError(errorMessage: string): boolean {
  return errorMessage.startsWith("PREMIUM_RESOURCE:");
}

/**
 * Extract premium plugin information from an error message
 * @param errorMessage The error message to parse
 * @returns An object with plugin id, version, and resource URL or null if parsing fails
 */
export function extractPremiumPluginInfo(errorMessage: string): { pluginId?: string, version?: string, resourceUrl: string } | null {
  if (!isPremiumPluginError(errorMessage)) {
    return null;
  }

  let resourceUrl = "";
  let pluginId = "";
  let version = "";

  // Handle the format with all parts (PREMIUM_RESOURCE:id:version:url)
  const parts = errorMessage.split(":");
  if (parts.length >= 4) {
    pluginId = parts[1];
    version = parts[2];
    resourceUrl = parts.slice(3).join(":"); // Rejoin in case URL contains colons
    return { pluginId, version, resourceUrl };
  }
  // Handle the simpler format (PREMIUM_RESOURCE:url)
  else if (parts.length >= 2) {
    resourceUrl = parts.slice(1).join(":"); // Get everything after PREMIUM_RESOURCE:
    return { resourceUrl };
  }

  return null;
}

/**
 * Check if a path string is empty or invalid
 * @param path The path to validate
 * @returns True if the path is valid, false otherwise
 */
export function isValidPath(path: string | undefined): boolean {
  return !!path && path.trim().length > 0;
}